/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef TRACE_H
#define TRACE_H

#include <string>
#include <map>
#include <inttypes.h>
#include <limits>
#include "TraceEvent.h"
#include "CallPathFilter.h"
#include "FlatFilter.h"
#include "SelectionFilter.h"
#include "ZoomFilter.h"
#include "RoleFilter.h"
#include "RegionPropertiesFilter.h"
#include "ResolutionFilter.h"
#include "LocationFilter.h"
#include "RegionClass.h"
#include "CallPathClass.h"
#include "LocationClass.h"
#include "LocationGroupClass.h"
#include "SystemTreeNodeClass.h"
#include "MetricMemberClass.h"
#include "MetricClassClass.h"
#include "MetricInstanceClass.h"

#include "MpiCollectiveGroup.h"
#include "MpiCollectiveGroupsCollection.h"
#include "MpiP2PGroup.h"
#include "MpiP2PGroupsCollection.h"
#include "OmpParallelGroup.h"
#include "OmpParallelGroupsCollection.h"

class Trace 
{
 
private:
    
    std::string tracepath;
    
    std::map<uint64_t, TraceEvent *> entries;
    uint64_t number_of_locations;
    uint64_t number_of_ranks;
    uint64_t start_timestamp;
    uint64_t end_timestamp;
    uint64_t timer_resolution;
    uint64_t global_offset;
    uint64_t trace_length;
    
    MpiCollectiveGroupsCollection * mpi_collective_groups;
    MpiP2PGroupsCollection * mpi_p2p_groups;
    OmpParallelGroupsCollection * omp_parallel_groups;
    
    std::vector<RegionClass> regions;
    std::vector<CallPathClass> calltree;
    
    std::vector< MetricClassClass > metric_classes;
    std::vector< MetricInstanceClass > metric_instances;
    std::vector< MetricMemberClass > metric_members;
    
    std::map<uint64_t, LocationClass> locations_classes;  
    std::map<uint64_t, LocationGroupClass> lg_classes;  
    std::map<uint64_t, SystemTreeNodeClass> stn_classes;  
    std::vector<uint64_t> root_stn_classes;  
    
    
    std::map<uint64_t,std::vector<TraceEvent* > > trace_view;
    std::map<uint64_t, int32_t> max_levels;
    
    bool callpath_filter_active;
    bool selection_filter_active;
    
    
public:

    Trace();
    ~Trace();
    
    void
    init(uint64_t, uint64_t, uint64_t, uint64_t, uint64_t, std::vector<RegionClass>&,  std::vector<CallPathClass>);
    
    void
    set_events(std::map<uint64_t, TraceEvent*>& );
    
    void
    set_max_levels( std::map<uint64_t, int32_t>);
         
    void
    set_system_tree(std::vector<uint64_t>&, 
                    std::map<uint64_t, SystemTreeNodeClass>&, 
                    std::map<uint64_t, LocationGroupClass>&, 
                    std::map<uint64_t, LocationClass>& );
    
    void
    set_metrics(std::vector< MetricMemberClass >& metric_members,
                std::vector< MetricClassClass >& metric_classes,
                std::vector< MetricInstanceClass >& metric_instances );   
    inline
    SystemTreeNodeClass& 
    get_system_tree_node( uint64_t _id) 
    {
            return stn_classes[_id];
    }
    
    inline
    std::vector<uint64_t>& 
    get_root_system_tree_nodes()
    {
            return root_stn_classes;
    }   

    
    inline
    LocationGroupClass& 
    get_location_group( uint64_t _id) 
    {
            return lg_classes[_id];
    }  
    
    inline
    LocationClass& 
    get_location( uint64_t _id)
    {
            return locations_classes[_id];
    }  
  
  
  
  
  
  
    inline
    std::map<uint64_t, LocationClass>& 
    get_locations()
    {
            return locations_classes;
    }  
  
    
    
    inline
    std::string
    get_system_tree_description()
    {
        std::string to_return;
        std::vector<uint64_t>& _r_stn = get_root_system_tree_nodes();
        for (size_t i=0; i<_r_stn.size(); ++i)
        {
            add_stn_description( to_return, get_system_tree_node(_r_stn[i]), 0);
        }
        return to_return;
    }
    
    
    inline
    void
    add_stn_description( std::string& description, SystemTreeNodeClass& stn, int level)
    {
        std::vector<uint64_t>& _child_stn = stn.children;
        std::vector<uint64_t>& _group_stn = stn.groups;
        for (int ind=0; ind < level; ++ind)
        {
            description+="       ";
        }
        description+=stn.name;
        for (size_t i=0; i<_child_stn.size(); ++i)
        {
            description+="\n";
            add_stn_description( description, get_system_tree_node(_child_stn[i]), level+1);
        }
        for (size_t i=0; i<_group_stn.size(); ++i)
        {
            description+="\n";
            add_location_group_description( description, get_location_group(_group_stn[i]), level+1);
        }
    }
    
    
    inline
    void
    add_location_group_description( std::string& description, LocationGroupClass& lg, int level)
    {
        for (int ind=0; ind < level; ++ind)
        {
            description+="       ";
        }
        description+=lg.name;
        std::vector<uint64_t>& _child_lg = lg.locations;
        for (size_t i=0; i<_child_lg.size(); ++i)
        {
            description+="\n";
            add_location_description( description, get_location(_child_lg[i]), level +1 );
        }
    }
    
    
    inline
    void
    add_location_description( std::string& description, LocationClass& loc, int level)
    {
        for (int ind=0; ind < level; ++ind)
        {
            description+="       ";
        }
        description+=loc.name;  
    }

    
    inline 
    bool
    is_callpath_filter_active()
    {
        return callpath_filter_active;
    }
    
    inline 
    bool
    is_selection_filter_active()
    {
        return selection_filter_active;
    }  
    
    inline 
    void
    set_callpath_filter_active(bool _ca)
    {
         callpath_filter_active = _ca;
    }
    
    inline 
    void
    set_selection_filter_active(bool _sa)
    {
        selection_filter_active = _sa;
    }  
       
    
    
    
    void
    print();
    
    void
    print(uint64_t);
    
    void
    clear_trace_view(std::vector<TraceEvent* >&);
    
    inline
    void
    set_trace_path( std::string& _path)
    {
        tracepath = _path;
    }
      
    inline
    std::string&
    get_trace_path()
    {
        return tracepath;
    }  
    
    inline
    int32_t
    get_max_level( uint64_t _location)
    {
        return max_levels[_location];
    }
    
    
    inline
    uint64_t
    get_start_timestamp()
    {
        return start_timestamp;
    }  
    
    inline
    uint64_t
    get_end_timestamp()
    {
        return end_timestamp;
    }
    
    inline
    uint64_t
    get_timer_resolution()
    {
        return timer_resolution;
    }  
    
    inline
    uint64_t
    get_trace_length()
    {
        return trace_length;
    }
      
    inline
    uint64_t
    get_global_offset()
    {
        return global_offset;
    }
        
    
    inline 
    MpiCollectiveGroupsCollection *
    get_mpi_collective_groups_collection() const
    {
        return mpi_collective_groups;
    }
    
    
    inline 
    void
    set_mpi_collective_groups_collection(MpiCollectiveGroupsCollection * _mgc)
    {
        mpi_collective_groups = _mgc;
    }   
    
    
    
   
    inline 
    MpiP2PGroupsCollection *
    get_mpi_p2p_groups_collection() const
    {
        return mpi_p2p_groups;
    }
    
    
    inline 
    void
    set_mpi_p2p_groups_collection(MpiP2PGroupsCollection * _mgc)
    {
        mpi_p2p_groups = _mgc;
    }   
      
    
    
    inline 
    OmpParallelGroupsCollection *
    get_omp_parallel_groups_collection() const
    {
        return omp_parallel_groups;
    }
    
    
    inline 
    void
    set_omp_parallel_groups_collection(OmpParallelGroupsCollection * _mgc)
    {
        omp_parallel_groups = _mgc;
    }   
         
    
    
    inline 
    uint64_t
    get_number_of_locations()
    {
        return number_of_locations;
    }
    
    
    inline 
    std::string
    get_region_name( uint64_t _r_id) const
    {
        if ( _r_id > regions.size())
            return "";
        
       return regions[_r_id].name;
    }
    
    inline
    size_t
    get_num_of_regions()const
    {
        return regions.size();
    }
    
    inline
    const RegionClass&
    get_region(size_t _id) const
    {
       return regions[_id];
    }  
    
    inline
    size_t
    get_num_of_metric_members()const
    {
        return metric_members.size();
    }
    
    inline
    const MetricMemberClass&
    get_metric_member(size_t _id) const
    {
       return metric_members[_id];
    }   
  
    inline
    size_t 
    get_metric_index(std::string & _name ) const
    {
        size_t i = 0;
        for (; i != metric_members.size(); ++i)
        {
            if (metric_members[i].name.compare(_name) == 0)
            {
                return i;
            }
        }
        return (size_t)-1;
    }   
  
  
  
    inline 
    OTF2_MetricMemberRef 
    get_metric_index( OTF2_MetricRef metric, size_t numberOfMetric) const
    {
        if (metric_instances.size() > metric)
        {
            const MetricClassClass&  mc =  metric_classes[metric_instances[metric].metricClass];
            size_t id = 0;
            for (id=0; id< mc.numberOfMetrics && id!= numberOfMetric; ++id)
            {}
            return mc.metricMembers[id];
        }
        
        
        const MetricClassClass&  mc =  metric_classes[metric];
        size_t id = 0;
        for (id=0; id< mc.numberOfMetrics && id!= numberOfMetric; ++id)
        {}
        return mc.metricMembers[id];
    }
  
  
  
  
  
    inline
    const std::vector<CallPathClass>&
    get_calltree() const 
    {
        return calltree;
    }
    
    inline
    uint64_t
    get_num_of_events() const
    {
        uint64_t _n_events=0;
        for (std::map<uint64_t, LocationClass>::const_iterator iter = locations_classes.begin(); iter != locations_classes.end(); ++iter)
        {
            _n_events+= (*iter).second.numberOfEvents;
        }
        return _n_events;        
    }
    
    inline 
    std::map<uint64_t, TraceEvent*>&
    get_location_entries()
    {
        return entries;
    }
    
    std::vector<TraceEvent* >&
    get_trace_view(uint64_t, ZoomFilter *, TraceEventFilter *, SelectionFilter * , ResolutionFilter *, LocationFilter *, RoleFilter *, RegionPropertiesFilter *);
    
    void
    add_to_view(std::vector<TraceEvent* >&trace_view, TraceEvent *_entry,  ZoomFilter *, TraceEventFilter *, SelectionFilter * , ResolutionFilter *, LocationFilter *, RoleFilter * , RegionPropertiesFilter * );
    
    
    std::pair<double, uint16_t> 
    convert_to_seconds( uint64_t ) const;
    
    
    
};

#endif // TRACELOADER_H
