/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef TRACE_EVENT_H
#define TRACE_EVENT_H

#include <string>
#include <vector>
#include <inttypes.h>
#include <otf2/otf2.h>
#include"TraceEventPoint.h"
#include "Cube.h"

class TraceEventGroup;

class TraceEvent 
{
protected:
    
    uint64_t location;
    TraceEvent * caller;
    
    
    std::vector<TraceEvent*> sub_calls;
    uint64_t start_timestamp, end_timestamp;
    uint64_t region;
    OTF2_Paradigm paradigm;
    OTF2_RegionRole role;
    bool visible;
    bool suppressed;  // display this event suppressed (in caseof trace events group display.. otehrwise ignored)
    bool leader;   // for operation reduce,gather and broadcast indicates being root
    
    uint32_t callpath;
    
    bool exclusive;
    int32_t level;
    
    cube::Location * cube_location;
    cube::Cnode * cube_callpath;
    
    std::vector<uint64_t> metric_start_value; // simple approach : id -> metric member id, value -> inclusive value of metric
    std::vector<uint64_t> metric_end_value; // simple approach : id -> metric member id, value -> inclusive value of metric
    
    
    
    TraceEventGroup * events_group;
    
    
public:
    TraceEvent( );
    TraceEvent(  uint64_t _location);
    virtual
    ~TraceEvent();
 
    
    inline 
    void
    set_events_group( TraceEventGroup *_c)
    {
        events_group = _c;
    }
    
    inline 
    TraceEventGroup *
    get_events_group() const
    {
        return events_group;
    }
    

    void
    set_suppressed( bool );
    
    inline
    bool
    is_suppressed() const
    {
        return suppressed;
    }
     
      
    void
    set_group_leader( bool );
    
    inline
    bool
    is_group_leader() const
    {
        return leader;
    }  
    
    
    inline
    uint64_t
    get_location() const
    {
        return location;
    }
    
    
    void
    set_level( int32_t _l);
    
    inline
    int32_t
    get_level() const
    {
        return level;
    }
    
    void
    set_visible( bool );
    
    inline
    bool
    is_visible() const
    {
        return visible;
    }
    

    inline
    void
    set_exclusive( bool _e )
    {
        exclusive = _e;
    }
    
    inline
    bool
    is_exclusive() const
    {
        return exclusive;
    }
    
    void
    set_start( uint64_t );
    
    void
    set_end( uint64_t );
    
    void
    set_start_metrics( std::vector<uint64_t>& );
    
    void
    set_end_metrics( std::vector<uint64_t>&  );
    
  
    const std::vector<uint64_t>&
    get_start_metrics( ) const ;

    const std::vector<uint64_t>& 
    get_end_metrics( ) const;
  
    
    
    
    
    void
    set_region( uint64_t );
    
    void
    set_paradigm( OTF2_Paradigm );
    
    OTF2_Paradigm 
    get_paradigm() const;
    
    void
    set_region_role( OTF2_RegionRole );
    
    OTF2_RegionRole 
    get_region_role() const;
    
    void
    set_callpath( uint32_t );
    
    inline
    uint64_t
    get_region() const
    {
        return region;
    }
    
    std::string
    get_region_name() const;
    
    
    
    inline
    uint64_t
    get_callpath( ) const
    {
        return callpath;
    }
    
    inline
    void
    set_parent( TraceEvent * _parent) 
    {
        caller = _parent;
        if (caller != NULL)
            caller->add_sub_trace_event( this );
    }
    
    
    inline
    TraceEvent*
    get_parent( ) const
    {
        return caller;
    }
    
    
    inline
    uint64_t
    get_start_timestamp( ) const
    {
        return start_timestamp;
    }
    
    inline
    uint64_t
    get_end_timestamp( ) const 
    {
        return end_timestamp;
    }
    
    inline
    void
    set_cube_location(cube::Location* _l) 
    {
        cube_location = _l;
    }    
    
    inline
    cube::Location *
    get_cube_location() const
    {
        return cube_location;
    }
    
    inline
    void
    set_cube_callpath(cube::Cnode* _c) 
    {
        cube_callpath = _c;
    }    
    
    inline
    cube::Cnode *
    get_cube_callpath() const
    {
        return cube_callpath;
    }   
    
    
    
    void
    add_sub_trace_event( TraceEvent * );
    
    void
    remove_child( TraceEvent * );
    
    
    void
    print( int );
    
    inline
    TraceEvent* 
    get_sub_call( size_t i)
    {
        return sub_calls[i];
    }
    
    inline
    size_t
    num_sub_calls()
    {
        return sub_calls.size();
    }
    
    std::pair< TraceEventPoint, TraceEventPoint> 
    get_trace_event_points();
    
    
    void
    reparent_children();
    
    
    
};


uint64_t location_extractor(const TraceEvent* _l);



#endif // TRACE_EVENT_H

